@@ -41,9 +41,15 @@
 #include <linux/magic.h>
 #include <limits.h>
 #include <stdarg.h>
+#include <sys/mman.h>
 #ifdef HAVE_LINUX_OPENAT2_H
 #  include <linux/openat2.h>
 #endif
+#if HAVE_STDATOMIC_H
+#  include <stdatomic.h>
+#else
+#  define atomic_long volatile long
+#endif
 
 #ifndef CLOSE_RANGE_CLOEXEC
 #  define CLOSE_RANGE_CLOEXEC (1U << 2)
@@ -265,10 +271,14 @@ ensure_directory_internal_at (int dirfd, char *path, size_t len, int mode, libcr
 
       if (parent_created || errno != ENOENT)
         {
+          libcrun_error_t tmp_err = NULL;
+
           /* On errors check if the directory already exists.  */
-          ret = crun_dir_p (path, false, err);
+          ret = crun_dir_p (path, false, &tmp_err);
           if (ret > 0)
             return 0;
+          if (ret < 0)
+            crun_error_release (&tmp_err);
 
           return crun_make_error (err, errno, "create directory `%s`", path);
         }
@@ -306,7 +316,7 @@ crun_ensure_directory_at (int dirfd, const char *path, int mode, bool nofollow,
     return ret;
 
   if (ret == 0)
-    return crun_make_error (err, ENOTDIR, "The path `%s` is not a directory", path);
+    return crun_make_error (err, ENOTDIR, "the path `%s` is not a directory", path);
 
   return 0;
 }
@@ -519,7 +529,7 @@ crun_safe_ensure_at (bool do_open, bool dir, int dirfd, const char *dirpath,
               /* If the previous openat fails, attempt to open the file in O_PATH mode.  */
               ret = openat (cwd, cur, O_CLOEXEC | O_PATH, 0);
               if (ret < 0)
-                return crun_make_error (err, errno, "open `%s/%s`", dirpath, cur);
+                return crun_make_error (err, errno, "open `%s/%s`", dirpath, npath);
             }
 
           if (do_open)
@@ -533,7 +543,7 @@ crun_safe_ensure_at (bool do_open, bool dir, int dirfd, const char *dirpath,
       if (ret < 0)
         {
           if (errno != EEXIST)
-            return crun_make_error (err, errno, "mkdir `%s`", cur);
+            return crun_make_error (err, errno, "mkdir `/%s`", npath);
         }
 
       cwd = safe_openat (dirfd, dirpath, dirpath_len, npath, O_CLOEXEC | O_PATH, 0, err);
@@ -752,7 +762,7 @@ static int
 libcrun_is_selinux_enabled (libcrun_error_t *err)
 {
   if (selinux_enabled < 0)
-    return crun_make_error (err, 0, "SELinux not initialized correctly");
+    return crun_make_error (err, 0, "SELinux is not initialized correctly");
   return selinux_enabled;
 }
 
@@ -804,7 +814,7 @@ write_file_and_check_fs_type (const char *file, const char *data, size_t len, un
 }
 
 int
-set_selinux_exec_label (const char *label, libcrun_error_t *err)
+set_selinux_label (const char *label, bool now, libcrun_error_t *err)
 {
   int ret;
 
@@ -814,7 +824,9 @@ set_selinux_exec_label (const char *label, libcrun_error_t *err)
 
   if (ret)
     {
-      ret = write_file_and_check_fs_type ("/proc/thread-self/attr/exec", label, strlen (label), PROC_SUPER_MAGIC,
+      const char *fname = now ? "/proc/thread-self/attr/current" : "/proc/thread-self/attr/exec";
+      ret = write_file_and_check_fs_type (fname, label,
+                                          strlen (label), PROC_SUPER_MAGIC,
                                           "procfs", err);
       if (UNLIKELY (ret < 0))
         return ret;
@@ -826,12 +838,12 @@ static int
 libcrun_is_apparmor_enabled (libcrun_error_t *err)
 {
   if (apparmor_enabled < 0)
-    return crun_make_error (err, 0, "AppArmor not initialized correctly");
+    return crun_make_error (err, 0, "AppArmor is not initialized correctly");
   return apparmor_enabled;
 }
 
 int
-set_apparmor_profile (const char *profile, libcrun_error_t *err)
+set_apparmor_profile (const char *profile, bool now, libcrun_error_t *err)
 {
   int ret;
 
@@ -840,11 +852,12 @@ set_apparmor_profile (const char *profile, libcrun_error_t *err)
     return ret;
   if (ret)
     {
+      const char *fname = now ? "/proc/thread-self/attr/current" : "/proc/thread-self/attr/exec";
       cleanup_free char *buf = NULL;
 
-      xasprintf (&buf, "exec %s", profile);
+      xasprintf (&buf, "%s %s", now ? "changeprofile" : "exec", profile);
 
-      ret = write_file_and_check_fs_type ("/proc/thread-self/attr/exec", buf, strlen (buf), PROC_SUPER_MAGIC, "procfs",
+      ret = write_file_and_check_fs_type (fname, buf, strlen (buf), PROC_SUPER_MAGIC, "procfs",
                                           err);
       if (UNLIKELY (ret < 0))
         return ret;
@@ -1140,8 +1153,10 @@ copy_from_fd_to_fd (int src, int dst, int consume, libcrun_error_t *err)
       cleanup_free char *buffer = NULL;
       ssize_t remaining;
 
+#define BUFFER_SIZE 4096
+
 #ifdef HAVE_COPY_FILE_RANGE
-      nread = copy_file_range (src, NULL, dst, NULL, 0, 0);
+      nread = copy_file_range (src, NULL, dst, NULL, BUFFER_SIZE, 0);
       if (nread < 0 && (errno == EINVAL || errno == EXDEV))
         goto fallback;
       if (consume && nread < 0 && errno == EAGAIN)
@@ -1150,10 +1165,10 @@ copy_from_fd_to_fd (int src, int dst, int consume, libcrun_error_t *err)
         return 0;
       if (UNLIKELY (nread < 0))
         return crun_make_error (err, errno, "copy_file_range");
+      continue;
 
     fallback:
 #endif
-#define BUFFER_SIZE 4096
 
       buffer = xmalloc (BUFFER_SIZE);
       nread = TEMP_FAILURE_RETRY (read (src, buffer, BUFFER_SIZE));
@@ -1167,7 +1182,7 @@ copy_from_fd_to_fd (int src, int dst, int consume, libcrun_error_t *err)
       remaining = nread;
       while (remaining)
         {
-          ret = TEMP_FAILURE_RETRY (write (dst, buffer, nread));
+          ret = TEMP_FAILURE_RETRY (write (dst, buffer + nread - remaining, remaining));
           if (UNLIKELY (ret < 0))
             return crun_make_error (err, errno, "write");
           remaining -= ret;
@@ -1393,7 +1408,7 @@ getsubidrange (uid_t id, int is_uid, uint32_t *from, uint32_t *len)
 #define MIN(x, y) ((x) < (y) ? (x) : (y))
 
 size_t
-format_default_id_mapping (char **ret, uid_t container_id, uid_t host_id, int is_uid)
+format_default_id_mapping (char **ret, uid_t container_id, uid_t host_uid, uid_t host_id, int is_uid)
 {
   uint32_t from, available;
   cleanup_free char *buffer = NULL;
@@ -1401,7 +1416,7 @@ format_default_id_mapping (char **ret, uid_t container_id, uid_t host_id, int is
 
   *ret = NULL;
 
-  if (getsubidrange (host_id, is_uid, &from, &available) < 0)
+  if (getsubidrange (host_uid, is_uid, &from, &available) < 0)
     return 0;
 
   /* More than enough space for all the mappings.  */
@@ -1427,17 +1442,65 @@ format_default_id_mapping (char **ret, uid_t container_id, uid_t host_id, int is
   return written;
 }
 
-/* will leave SIGCHLD blocked if TIMEOUT is used.  */
+static void __attribute__ ((__noreturn__))
+run_process_child (char *path, char **args, const char *cwd, char **envp, int pipe_r,
+                   int pipe_w, int out_fd, int err_fd)
+{
+  char *tmp_args[] = { path, NULL };
+  libcrun_error_t err = NULL;
+  int dev_null_fd = -1;
+  int ret;
+
+  ret = mark_or_close_fds_ge_than (3, false, &err);
+  if (UNLIKELY (ret < 0))
+    libcrun_fail_with_error ((err)->status, "%s", (err)->msg);
+
+  if (out_fd < 0 || err_fd < 0)
+    {
+      dev_null_fd = open ("/dev/null", O_WRONLY);
+      if (UNLIKELY (dev_null_fd < 0))
+        _exit (EXIT_FAILURE);
+    }
+
+  TEMP_FAILURE_RETRY (close (pipe_w));
+  dup2 (pipe_r, 0);
+  TEMP_FAILURE_RETRY (close (pipe_r));
+
+  dup2 (out_fd >= 0 ? out_fd : dev_null_fd, 1);
+  dup2 (err_fd >= 0 ? err_fd : dev_null_fd, 2);
+
+  if (dev_null_fd >= 0)
+    TEMP_FAILURE_RETRY (close (dev_null_fd));
+  if (out_fd >= 0)
+    TEMP_FAILURE_RETRY (close (out_fd));
+  if (err_fd >= 0)
+    TEMP_FAILURE_RETRY (close (err_fd));
+
+  if (args == NULL)
+    args = tmp_args;
+
+  if (cwd && chdir (cwd) < 0)
+    _exit (EXIT_FAILURE);
+
+  execvpe (path, args, envp);
+  _exit (EXIT_FAILURE);
+}
+
+/* It changes the signals mask for the current process.  */
 int
-run_process_with_stdin_timeout_envp (char *path, char **args, const char *cwd, int timeout, char **envp, char *stdin,
-                                     size_t stdin_len, int out_fd, int err_fd, libcrun_error_t *err)
+run_process_with_stdin_timeout_envp (char *path, char **args, const char *cwd, int timeout,
+                                     char **envp, char *stdin, size_t stdin_len, int out_fd,
+                                     int err_fd, libcrun_error_t *err)
 {
   int stdin_pipe[2];
   pid_t pid;
   int ret;
   cleanup_close int pipe_r = -1;
   cleanup_close int pipe_w = -1;
-  sigset_t mask;
+  sigset_t oldmask, mask;
+  int r, status;
+
+  sigemptyset (&mask);
 
   ret = pipe (stdin_pipe);
   if (UNLIKELY (ret < 0))
@@ -1448,101 +1511,89 @@ run_process_with_stdin_timeout_envp (char *path, char **args, const char *cwd, i
   if (timeout > 0)
     {
       sigaddset (&mask, SIGCHLD);
-      ret = sigprocmask (SIG_BLOCK, &mask, NULL);
+      ret = sigprocmask (SIG_BLOCK, &mask, &oldmask);
       if (UNLIKELY (ret < 0))
         return crun_make_error (err, errno, "sigprocmask");
     }
 
   pid = fork ();
   if (UNLIKELY (pid < 0))
-    return crun_make_error (err, errno, "fork");
-
-  if (pid)
     {
-      int r, status;
+      ret = crun_make_error (err, errno, "fork");
+      goto restore_sig_mask_and_exit;
+    }
 
-      close_and_reset (&pipe_r);
+  if (pid == 0)
+    {
+      /* run_process_child doesn't return.  */
+      run_process_child (path, args, cwd, envp, pipe_r, pipe_w, out_fd, err_fd);
+    }
 
-      ret = TEMP_FAILURE_RETRY (write (pipe_w, stdin, stdin_len));
-      if (UNLIKELY (ret < 0))
-        return crun_make_error (err, errno, "writing to pipe");
+  close_and_reset (&pipe_r);
 
-      close_and_reset (&pipe_w);
+  ret = TEMP_FAILURE_RETRY (write (pipe_w, stdin, stdin_len));
+  if (UNLIKELY (ret < 0))
+    {
+      ret = crun_make_error (err, errno, "writing to pipe");
+      goto restore_sig_mask_and_exit;
+    }
 
-      if (timeout)
+  close_and_reset (&pipe_w);
+
+  if (timeout)
+    {
+      time_t start = time (NULL);
+      time_t now;
+      for (now = start; now - start < timeout; now = time (NULL))
         {
-          time_t start = time (NULL);
-          time_t now;
-          for (now = start; now - start < timeout; now = time (NULL))
+          siginfo_t info;
+          int elapsed = now - start;
+          struct timespec ts_timeout = { .tv_sec = timeout - elapsed, .tv_nsec = 0 };
+
+          ret = sigtimedwait (&mask, &info, &ts_timeout);
+          if (UNLIKELY (ret < 0 && errno != EAGAIN))
             {
-              siginfo_t info;
-              int elapsed = now - start;
-              struct timespec ts_timeout = { .tv_sec = timeout - elapsed, .tv_nsec = 0 };
-
-              ret = sigtimedwait (&mask, &info, &ts_timeout);
-              if (UNLIKELY (ret < 0 && errno != EAGAIN))
-                return crun_make_error (err, errno, "sigtimedwait");
-
-              if (info.si_signo == SIGCHLD && info.si_pid == pid)
-                goto read_waitpid;
-
-              if (ret < 0 && errno == EAGAIN)
-                goto timeout;
+              ret = crun_make_error (err, errno, "sigtimedwait");
+              goto restore_sig_mask_and_exit;
             }
-        timeout:
-          kill (pid, SIGKILL);
-          return crun_make_error (err, 0, "timeout expired for `%s`", path);
-        }
 
-    read_waitpid:
-      r = TEMP_FAILURE_RETRY (waitpid (pid, &status, 0));
-      if (r < 0)
-        return crun_make_error (err, errno, "waitpid");
-      if (WIFEXITED (status))
-        return WEXITSTATUS (status);
-      if (WIFSIGNALED (status))
-        return 127 + WTERMSIG (status);
+          if (info.si_signo == SIGCHLD && info.si_pid == pid)
+            goto read_waitpid;
+
+          if (ret < 0 && errno == EAGAIN)
+            goto timeout;
+        }
+    timeout:
+      kill (pid, SIGKILL);
+
+      ret = crun_make_error (err, 0, "timeout expired for `%s`", path);
+      goto restore_sig_mask_and_exit;
     }
+
+read_waitpid:
+  r = waitpid_ignore_stopped (pid, &status, 0);
+  if (r < 0)
+    ret = crun_make_error (err, errno, "waitpid");
   else
+    ret = get_process_exit_status (status);
+
+  /* Prevent to cleanup the pid again.  */
+  pid = 0;
+
+restore_sig_mask_and_exit:
+  if (timeout > 0)
     {
-      char *tmp_args[] = { path, NULL };
-      int dev_null_fd = -1;
-
-      ret = mark_or_close_fds_ge_than (3, false, err);
-      if (UNLIKELY (ret < 0))
-        libcrun_fail_with_error ((*err)->status, "%s", (*err)->msg);
-
-      if (out_fd < 0 || err_fd < 0)
+      /* Cleanup the zombie process.  */
+      if (pid > 0)
         {
-          dev_null_fd = open ("/dev/null", O_WRONLY);
-          if (UNLIKELY (dev_null_fd < 0))
-            _exit (EXIT_FAILURE);
+          kill (pid, SIGKILL);
+          TEMP_FAILURE_RETRY (waitpid (pid, &status, 0));
         }
-
-      TEMP_FAILURE_RETRY (close (pipe_w));
-      dup2 (pipe_r, 0);
-      TEMP_FAILURE_RETRY (close (pipe_r));
-
-      dup2 (out_fd >= 0 ? out_fd : dev_null_fd, 1);
-      dup2 (err_fd >= 0 ? err_fd : dev_null_fd, 2);
-
-      if (dev_null_fd >= 0)
-        TEMP_FAILURE_RETRY (close (dev_null_fd));
-      if (out_fd >= 0)
-        TEMP_FAILURE_RETRY (close (out_fd));
-      if (err_fd >= 0)
-        TEMP_FAILURE_RETRY (close (err_fd));
-
-      if (args == NULL)
-        args = tmp_args;
-
-      if (cwd && chdir (cwd) < 0)
-        _exit (EXIT_FAILURE);
-
-      execvpe (path, args, envp);
-      _exit (EXIT_FAILURE);
+      r = sigprocmask (SIG_UNBLOCK, &oldmask, NULL);
+      if (UNLIKELY (r < 0 && ret >= 0))
+        ret = crun_make_error (err, errno, "restoring signal mask with sigprocmask");
     }
-  return -1;
+  return ret;
 }
 
 int
@@ -1618,7 +1669,7 @@ get_current_timestamp (char *out, size_t len)
   gmtime_r (&tv.tv_sec, &now);
   strftime (timestamp, sizeof (timestamp), "%Y-%m-%dT%H:%M:%S", &now);
 
-  snprintf (out, len, "%s.%09ldZ", timestamp, tv.tv_usec);
+  snprintf (out, len, "%s.%06ldZ", timestamp, tv.tv_usec);
   out[len - 1] = '\0';
 }
 
@@ -1649,13 +1700,6 @@ parse_json_file (yajl_val *out, const char *jsondata, struct parser_context *ctx
   return 0;
 }
 
-int
-has_prefix (const char *str, const char *prefix)
-{
-  size_t prefix_len = strlen (prefix);
-  return strlen (str) >= prefix_len && memcmp (str, prefix, prefix_len) == 0;
-}
-
 static int
 check_access (const char *path)
 {
@@ -1926,7 +1970,7 @@ copy_recursive_fd_to_fd (int srcdirfd, int dfd, const char *srcname, const char
           if (UNLIKELY (destfd < 0))
             return crun_make_error (err, errno, "open `%s/%s`", destname, de->d_name);
 
-          ret = copy_from_fd_to_fd (srcfd, destfd, 0, err);
+          ret = copy_from_fd_to_fd (srcfd, destfd, 1, err);
           if (UNLIKELY (ret < 0))
             return ret;
 
@@ -2186,7 +2230,7 @@ base64_decode (const char *iptr, size_t isize, char *optr, size_t osize, size_t
             }
         }
 
-      /* I have a entire block of data 32 bits get the output data.  */
+      /* I have an entire block of data 32 bits get the output data.  */
       if (i == 4)
         {
           *optr++ = (data[0] << 2) | ((data[1] & 0x30) >> 4);
@@ -2253,3 +2297,90 @@ str_join_array (int offset, size_t size, char *const array[], const char *joint)
   *p = '\0';
   return result;
 }
+
+int
+libcrun_mmap (struct libcrun_mmap_s **ret, void *addr, size_t length,
+              int prot, int flags, int fd, off_t offset,
+              libcrun_error_t *err)
+{
+  struct libcrun_mmap_s *mmap_s = NULL;
+
+  void *mapped = mmap (addr, length, prot, flags, fd, offset);
+  if (mapped == MAP_FAILED)
+    return crun_make_error (err, errno, "mmap");
+
+  mmap_s = xmalloc (sizeof (struct libcrun_mmap_s));
+  mmap_s->addr = mapped;
+  mmap_s->length = length;
+
+  *ret = mmap_s;
+
+  return 0;
+}
+
+int
+libcrun_munmap (struct libcrun_mmap_s *mmap, libcrun_error_t *err)
+{
+  int ret;
+
+  ret = munmap (mmap->addr, mmap->length);
+  if (UNLIKELY (ret < 0))
+    return crun_make_error (err, errno, "munmap");
+
+  free (mmap);
+
+  return 0;
+}
+
+static long
+read_file_as_long_or_default (const char *path, long def_value)
+{
+  cleanup_free char *content = NULL;
+  libcrun_error_t tmp_err = NULL;
+  char *endptr;
+  long val;
+  int ret;
+
+  ret = read_all_file (path, &content, NULL, &tmp_err);
+  if (UNLIKELY (ret < 0))
+    {
+      crun_error_release (&tmp_err);
+      return def_value;
+    }
+
+  errno = 0;
+  val = strtol (content, &endptr, 10);
+  if (UNLIKELY (errno))
+    return def_value;
+  if (endptr == content || (*endptr && *endptr != '\n'))
+    return def_value;
+  return val;
+}
+
+#define DEFAULT_OVERFLOW_ID 65534
+
+uid_t
+get_overflow_uid (void)
+{
+  static atomic_long cached_uid = -1;
+  atomic_long uid = cached_uid;
+  if (uid == -1)
+    {
+      uid = read_file_as_long_or_default ("/proc/sys/kernel/overflowuid", DEFAULT_OVERFLOW_ID);
+      cached_uid = uid;
+    }
+  return uid;
+}
+
+gid_t
+get_overflow_gid (void)
+{
+  static atomic_long cached_gid = -1;
+  atomic_long gid = cached_gid;
+  if (gid == -1)
+    {
+      gid = read_file_as_long_or_default ("/proc/sys/kernel/overflowgid", DEFAULT_OVERFLOW_ID);
+      cached_gid = gid;
+    }
+  return gid;
+}
