@@ -27,13 +27,13 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <signal.h>
-#include <runtime_spec_schema_config_schema.h>
+#include <ocispec/runtime_spec_schema_config_schema.h>
 #include <sys/wait.h>
 #include "container.h"
 
 #ifndef TEMP_FAILURE_RETRY
 #  define TEMP_FAILURE_RETRY(expression)      \
-    (__extension__({                          \
+    (__extension__ ({                         \
       long int __result;                      \
       do                                      \
         __result = (long int) (expression);   \
@@ -50,6 +50,7 @@
 #define cleanup_dir __attribute__ ((cleanup (cleanup_dirp)))
 #define arg_unused __attribute__ ((unused))
 #define cleanup_pid __attribute__ ((cleanup (cleanup_pidp)))
+#define cleanup_mmap __attribute__ ((cleanup (cleanup_mmapp)))
 
 #define LIKELY(x) __builtin_expect ((x), 1)
 #define UNLIKELY(x) __builtin_expect ((x), 0)
@@ -115,6 +116,34 @@ cleanup_pidp (void *p)
     }
 }
 
+struct libcrun_mmap_s
+{
+  void *addr;
+  size_t length;
+};
+
+int libcrun_mmap (struct libcrun_mmap_s **ret, void *addr, size_t length,
+                  int prot, int flags, int fd, off_t offset,
+                  libcrun_error_t *err);
+
+int libcrun_munmap (struct libcrun_mmap_s *mmap, libcrun_error_t *err);
+
+static inline void
+cleanup_mmapp (void *p)
+{
+  int ret;
+  libcrun_error_t tmp_err = NULL;
+  struct libcrun_mmap_s **mm;
+
+  mm = (struct libcrun_mmap_s **) p;
+  if (*mm == NULL)
+    return;
+
+  ret = libcrun_munmap (*mm, &tmp_err);
+  if (UNLIKELY (ret < 0))
+    crun_error_release (&tmp_err);
+}
+
 struct libcrun_fd_map
 {
   size_t nfds;
@@ -260,11 +289,11 @@ int create_file_if_missing_at (int dirfd, const char *file, libcrun_error_t *err
 
 int check_running_in_user_namespace (libcrun_error_t *err);
 
-int set_selinux_exec_label (const char *label, libcrun_error_t *err);
+int set_selinux_label (const char *label, bool now, libcrun_error_t *err);
 
 int add_selinux_mount_label (char **ret, const char *data, const char *label, libcrun_error_t *err);
 
-int set_apparmor_profile (const char *profile, libcrun_error_t *err);
+int set_apparmor_profile (const char *profile, bool now, libcrun_error_t *err);
 
 int read_all_fd (int fd, const char *description, char **out, size_t *len, libcrun_error_t *err);
 
@@ -294,7 +323,7 @@ int copy_from_fd_to_fd (int src, int dst, int consume, libcrun_error_t *err);
 
 int run_process (char **args, libcrun_error_t *err);
 
-size_t format_default_id_mapping (char **ret, uid_t container_id, uid_t host_id, int is_uid);
+size_t format_default_id_mapping (char **ret, uid_t container_id, uid_t host_uid, uid_t host_id, int is_uid);
 
 int run_process_with_stdin_timeout_envp (char *path, char **args, const char *cwd, int timeout, char **envp,
                                          char *stdin, size_t stdin_len, int out_fd, int err_fd, libcrun_error_t *err);
@@ -307,7 +336,12 @@ int set_blocking_fd (int fd, int blocking, libcrun_error_t *err);
 
 int parse_json_file (yajl_val *out, const char *jsondata, struct parser_context *ctx, libcrun_error_t *err);
 
-int has_prefix (const char *str, const char *prefix);
+static inline int
+has_prefix (const char *str, const char *prefix)
+{
+  size_t prefix_len = strlen (prefix);
+  return strlen (str) >= prefix_len && memcmp (str, prefix, prefix_len) == 0;
+}
 
 char *find_executable (const char *executable_path, const char *cwd);
 
@@ -349,4 +383,35 @@ is_empty_string (const char *s)
   return s == NULL || s[0] == '\0';
 }
 
+static inline int
+waitpid_ignore_stopped (pid_t pid, int *status, int options)
+{
+  int ret, s = 0;
+  do
+    {
+      ret = TEMP_FAILURE_RETRY (waitpid (pid, &s, options));
+      if (ret < 0)
+        return ret;
+  } while (WIFSTOPPED (s) | WIFCONTINUED (s));
+
+  if (status)
+    *status = s;
+
+  return ret;
+}
+
+static inline int
+get_process_exit_status (int status)
+{
+  if (WIFEXITED (status))
+    return WEXITSTATUS (status);
+  if (WIFSIGNALED (status))
+    return 128 + WTERMSIG (status);
+
+  return -1;
+}
+
+uid_t get_overflow_uid (void);
+gid_t get_overflow_gid (void);
+
 #endif
